import fastifyPlugin from 'fastify-plugin';
// 雪花ID
class Snowflake {
    constructor({ workerId = 0n, dataCenterId = 0n, sequence = 0n } = {}) {
        this.workerIdBits = 3n; // 工作节点ID位数
        this.dataCenterIdBits = 6n; // 数据中心ID位数
        this.sequenceBits = 12n; // 序列号位数

        this.maxWorkerId = -1n ^ (-1n << this.workerIdBits);
        this.maxDataCenterId = -1n ^ (-1n << this.dataCenterIdBits);

        this.workerIdShift = this.sequenceBits;
        this.dataCenterIdShift = this.sequenceBits + this.workerIdBits;
        this.timestampShift = this.sequenceBits + this.workerIdBits + this.dataCenterIdBits;
        this.sequenceMask = -1n ^ (-1n << this.sequenceBits);

        this.epoch = 1577836800000n; // 2020-01-01 00:00:00 UTC

        this.lastTimestamp = -1n;
        this.sequence = sequence;
        this.workerId = BigInt(workerId);
        this.dataCenterId = BigInt(dataCenterId);

        if (this.workerId > this.maxWorkerId || this.workerId < 0n) {
            throw new Error(`Worker ID must be between 0 and ${this.maxWorkerId}`);
        }
        if (this.dataCenterId > this.maxDataCenterId || this.dataCenterId < 0n) {
            throw new Error(`Data Center ID must be between 0 and ${this.maxDataCenterId}`);
        }
    }

    nextId() {
        let timestamp = BigInt(Date.now());

        if (timestamp < this.lastTimestamp) {
            throw new Error('Clock moved backwards!');
        }

        if (this.lastTimestamp === timestamp) {
            this.sequence = (this.sequence + 1n) & this.sequenceMask;
            if (this.sequence === 0n) {
                while (timestamp <= this.lastTimestamp) {
                    timestamp = BigInt(Date.now());
                }
            }
        } else {
            this.sequence = 0n;
        }

        this.lastTimestamp = timestamp;

        return (
            ((timestamp - this.epoch) << this.timestampShift) |
            (this.dataCenterId << this.dataCenterIdShift) |
            (this.workerId << this.workerIdShift) |
            this.sequence
        );
    }
}
async function snowflake(fastify, options) {
    // 注册swagger插件（新版配置方式）
    fastify.log.warn('Register Snowflake Plugin!');
    // 初始化生成器（工作节点ID范围 0-31）
    const generator = new Snowflake({
        workerId: BigInt(fastify.config.deviceInfo.machineId), // 工作节点ID
        dataCenterId: BigInt(fastify.config.deviceInfo.dataCenterId), // 数据中心ID
    });
    // 初始化时注册插件
    fastify.decorate('snowflake', () => generator.nextId().toString());
    fastify.log.warn('Register Snowflake complete!');
}

export default fastifyPlugin(snowflake);
