import { checkExistsUsernameAndEmail, registerAccountForMyself } from '#services/user/user.db';
// 新用户注册
export async function userRegisterService(request, reply) {
    // 1. 获取已验证的参数
    const { username, email, password } = request.body;

    // 2. 分布式锁
    // 生成分布式锁 key
    const lockKey = `${this.const.DISTRIBUTED_LOCK_PREFIX.REGISTER_USER}${username}:${email}`;

    const userId = this.snowflake();
    const lockIdentifier = userId;
    let renewInterval;

    try {
        // 尝试获取分布式锁（NX: 仅当key不存在时设置，EX: 过期时间5秒）
        const locked = await this.redis.SET(lockKey, lockIdentifier, { NX: true, EX: 5 });
        if (!locked) {
            throw this.httpErrors.tooManyRequests('操作正在进行，请稍后重试');
        }

        // 启动续锁定时器（每3秒续期一次）
        renewInterval = setInterval(async () => {
            const currentVal = await this.redis.GET(lockKey);
            if (currentVal === lockIdentifier) {
                await this.redis.EXPIRE(lockKey, 5); // 重置过期时间
            }
        }, 3000);
        await new Promise(resolve => setTimeout(resolve, 3000));
        // 2. 检测用户名和邮箱是否已存在
        const exists = await checkExistsUsernameAndEmail.call(this, username, email);
        if (exists) {
            throw this.httpErrors.conflict('用户名或邮箱已存在');
        }

        // 3. 注册用户
        const newUser = await registerAccountForMyself.call(this, {
            username,
            email,
            password: await this.hash(password),
            userId
        });
        reply.code(201) // <-- 关键修改
            .send({
                code: 201,
                data: newUser,
                message: '用户创建成功'
            });
    } finally {
        if (renewInterval) clearInterval(renewInterval);
        // 释放锁逻辑（移除Lua脚本）
        const currentVal = await this.redis.GET(lockKey);
        if (currentVal === lockIdentifier) {
            await this.redis.DEL(lockKey);
        }
    }


}
